use crate::vulkan::{VkDevice, VkInstance, VkPhysicalDevice, VkSurface, VkSwapchain};
use ash::vk::QueueFlags;

pub struct VkRenderContext {
    instance: VkInstance,
    surface: VkSurface,
    device: VkDevice,
    swapchain: VkSwapchain
}

impl VkRenderContext {
    pub fn init(window: &crate::display::Window) -> anyhow::Result<Self> {
        let required_extensions = window
            .required_extensions()?;

        let instance = VkInstance::new(&required_extensions);
        let surface = instance.create_surface(&window)?;

        let mut physical_devices = instance.get_physical_devices();
        physical_devices.sort_by(|a, b| b.priority().cmp(&a.priority()));

        let (physical_device, queue_family_index, _) = VkPhysicalDevice::pick_physical_device_and_queue_by(
            &physical_devices,
            Some(QueueFlags::GRAPHICS),
            Some(&surface),
        ).expect("Unable to find physical device");

        let device = VkDevice::new_graphics_device(&instance, &physical_device, queue_family_index)
            .expect("Unable to create device");

        let swapchain = surface.create_swapchain(
            &window,
            &instance,
            &device,
            &physical_device
        ).expect("Unable to create swapchain");

        // let present_queue = device.get_device_queue(0);

        Ok(Self {
            instance,
            surface,
            device,
            swapchain
        })
    }
}